/*
 *  TOPPERS Software
 *      Toyohashi Open Platform for Embedded Real-Time Systems
 *
 *  Copyright (C) 2007-2008 by TAKAGI Nobuhisa
 * 
 *  嵭Ԥϡʲ(1)(4)ξ˸¤ꡤܥեȥ
 *  ܥեȥѤΤޤࡥʲƱˤѡʣ
 *  ѡۡʰʲѤȸƤ֡ˤ뤳Ȥ̵ǵ롥
 *  (1) ܥեȥ򥽡ɤηѤˤϡ嵭
 *      ɽѾ浪Ӳ̵ݾڵ꤬Τޤޤηǥ
 *      ˴ޤޤƤ뤳ȡ
 *  (2) ܥեȥ򡤥饤֥ʤɡ¾Υեȥȯ˻
 *      ѤǤǺۤˤϡۤȼɥȡ
 *      ԥޥ˥奢ʤɡˤˡ嵭ɽѾ浪Ӳ
 *      ̵ݾڵǺܤ뤳ȡ
 *  (3) ܥեȥ򡤵Ȥ߹ʤɡ¾Υեȥȯ˻
 *      ѤǤʤǺۤˤϡΤ줫ξ
 *      ȡ
 *    (a) ۤȼɥȡѼԥޥ˥奢ʤɡˤˡ嵭
 *        ɽѾ浪Ӳ̵ݾڵǺܤ뤳ȡ
 *    (b) ۤη֤̤ˡˤäơTOPPERSץȤ
 *        𤹤뤳ȡ
 *  (4) ܥեȥѤˤľŪޤϴŪ뤤ʤ»
 *      ⡤嵭ԤTOPPERSץȤդ뤳ȡ
 *      ޤܥեȥΥ桼ޤϥɥ桼Τʤ
 *      ͳ˴Ťᤫ⡤嵭ԤTOPPERSץȤ
 *      դ뤳ȡ
 * 
 *  ܥեȥϡ̵ݾڤ󶡤ƤΤǤ롥嵭Ԥ
 *  TOPPERSץȤϡܥեȥ˴ؤơλŪ
 *  ФŬޤơʤݾڤԤʤޤܥեȥ
 *  ѤˤľŪޤϴŪʤ»˴ؤƤ⡤
 *  Ǥʤ
 * 
 */
#include <cstdio>
#include <string>
#include "toppers/diagnostics.hpp"
#include "toppers/text_line.hpp"

namespace toppers
{
  namespace
  {
    int error_abort_threshold = 100;
    int error_count = 0;
    std::string program_name( "(unknown)" );
  }

  int get_error_count()
  {
    return error_count;
  }

  int increment_error_count()
  {
    return ++error_count;
  }

  void set_program_name( char const* name )
  {
    program_name = name;
  }

  std::string const& get_program_name()
  {
    return program_name;
  }

  int set_error_abort_threshold( int thresh )
  {
    if ( thresh < 1 )
    {
      return -1;
    }
    int previous = error_abort_threshold;
    error_abort_threshold = thresh;
    return previous;
  }

  void warning( const char* msg )
  {
    fprintf( stderr, "%s: %s: %s\n", program_name.c_str(), _( "warning" ), msg );
  }
  void warning( text_line const& line, const char* msg )
  {
    fprintf( stderr, "%s:%s:%ld: %s: %s\n", program_name.c_str(), line.file.c_str(), line.line, _( "warning" ), msg );
  }
  void error( const char* msg )
  {
    fprintf( stderr, "%s: %s: %s\n", program_name.c_str(), _( "error" ), msg );
    ++error_count;
    if ( error_abort_threshold <= error_count )
    {
      throw diagnostics_error( _( "too many errors" ) );
    }
  }
  void error( text_line const& line, const char* msg )
  {
    fprintf( stderr, "%s:%s:%ld: %s: %s\n", program_name.c_str(), line.file.c_str(), line.line, _( "error" ), msg );
    ++error_count;
    if ( error_abort_threshold <= error_count )
    {
      throw diagnostics_error( _( "too many errors" ) );
    }
  }
  void fatal( const char* msg )
  {
    fprintf( stderr, "%s: %s: %s\n", program_name.c_str(), _( "error" ), msg );
    throw diagnostics_error( _( "fatal error" ) );
  }
  void fatal( text_line const& line, const char* msg )
  {
    fprintf( stderr, "%s:%s:%d: %s: %s\n", program_name.c_str(), line.file.c_str(), line.line, _( "error" ), msg );
    throw diagnostics_error( _( "fatal error" ) );
  }

}
